/*
 * ========================================================================
 * dynfcn.c - Robot dynamics computations.
 *
 * Computational function that takes tau, q, dq, and the dynamic model
 * parameters to compute ddq.
 *
 * This is a MEX-file for MATLAB.
 * ========================================================================
 */

#include <stdarg.h> 
#include <math.h>
#include <time.h>
#include <sys/times.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
//#include "mex.h"
#include "dynfcn.h"

#define g gconst

/*-----------------------------------------------------------------------*/
/*                Power()                                                */
/*-----------------------------------------------------------------------*/
double Power(double x, int n){

  int i;
  double aux;

  for(i=0, aux=1.0; i<n ; aux*=x, i++) ;    

  return aux;

}

/*-----------------------------------------------------------------------*/
/*                ListVector()                                           */
/*-----------------------------------------------------------------------*/
int ListVector(int ncols, double **vector, ...){

  va_list ap;
  int j;
  double *vec;

  vec = *vector;

  va_start(ap,vector);

  for(j=0 ; j<ncols ; j++)
    vec[j]=va_arg(ap,double);

  va_end(ap);

  *vector -= ncols ;

  return 0;

}

/*-----------------------------------------------------------------------*/
/*                ListMatrix()                                           */
/*-----------------------------------------------------------------------*/
void ListMatrix(int nrows, ...){

  return;

}

/*-----------------------------------------------------------------------*/
/*                mexFunction()                                          */
/*-----------------------------------------------------------------------*/
void calcMatrix(int dof, vect_n *jAng, matr_mn *M, vect_n *G, matr_mn *Js, matr_mn *Th){

  double *inp, *out;
  int mrows, ncols;
  double MassMatrixVec[8*8];
  double *MassMatrix[8];
  double CoriolisMatrixVec[8*8];
  double *CoriolisMatrix[8];
  double CoriolisTermVec[8*1];
  double *CoriolisTerm[8];
  double GravityTermVec[8*1];
  double *GravityTerm[8];
  double *aux1, *aux2, *aux3, *aux4;
  double *ListMatrixPtr;
  double **ListMatrixDPtr;
  double tau[8];
  //double q[8];
  double *q;
  double dq[8];
  double ddq[8];
  int ListMatrixRows;
  int ListMatrixCols;
  int i, j, k;
  //int dof;
  gsl_matrix_view gslMassMatrix;
  gsl_vector_view b_side;
  gsl_vector *gsl_ddq;
  int signum;
  gsl_permutation *permut;
  double JvcVec[8*8*8];
  gsl_matrix_view gslJvc[8];
  double JwcVec[8*8*8];
  gsl_matrix_view gslJwc[8];
  double RotVec[8*3*3];
  gsl_matrix_view gslRot[8];
  double ThomVec[8*4*4];
  gsl_matrix_view gslThom[8];
  int ListMatrixIndex;
  int aux;
  double auxm1Vec[8*8];
  gsl_matrix_view auxm1;
  double auxm2Vec[8*8];
  gsl_matrix_view auxm2;
  double auxm3Vec[8*8];
  gsl_matrix_view auxm3;
  #include "robot_Spec.h"
  double mass[7] = { mass1, mass2, mass3, mass4, mass5, mass6, mass7 };
  double inertia[7][3][3]={
    {
      {Lxx1,Lxy1,Lxz1},
      {Lyx1,Lyy1,Lyz1},
      {Lzx1,Lzy1,Lzz1}
    },
    {
      {Lxx2,Lxy2,Lxz2},
      {Lyx2,Lyy2,Lyz2},
      {Lzx2,Lzy2,Lzz2}
    },
    {
      {Lxx3,Lxy3,Lxz3},
      {Lyx3,Lyy3,Lyz3},
      {Lzx3,Lzy3,Lzz3}
    },
    {
      {Lxx4,Lxy4,Lxz4},
      {Lyx4,Lyy4,Lyz4},
      {Lzx4,Lzy4,Lzz4}
    },
    {
      {Lxx5,Lxy5,Lxz5},
      {Lyx5,Lyy5,Lyz5},
      {Lzx5,Lzy5,Lzz5}
    },
    {
      {Lxx6,Lxy6,Lxz6},
      {Lyx6,Lyy6,Lyz6},
      {Lzx6,Lzy6,Lzz6}
    },
    {
      {Lxx7,Lxy7,Lxz7},
      {Lyx7,Lyy7,Lyz7},
      {Lzx7,Lzy7,Lzz7}
    }
  };

#if 0
  double inertia[7][3][3]={
    {
      {Ixx1,Ixy1,Ixz1},
      {Iyx1,Iyy1,Iyz1},
      {Izx1,Izy1,Izz1}
    },
    {
      {Ixx2,Ixy2,Ixz2},
      {Iyx2,Iyy2,Iyz2},
      {Izx2,Izy2,Izz2}
    },
    {
      {Ixx3,Ixy3,Ixz3},
      {Iyx3,Iyy3,Iyz3},
      {Izx3,Izy3,Izz3}
    },
    {
      {Ixx4,Ixy4,Ixz4},
      {Iyx4,Iyy4,Iyz4},
      {Izx4,Izy4,Izz4}
    },
    {
      {Ixx5,Ixy5,Ixz5},
      {Iyx5,Iyy5,Iyz5},
      {Izx5,Izy5,Izz5}
    },
    {
      {Ixx6,Ixy6,Ixz6},
      {Iyx6,Iyy6,Iyz6},
      {Izx6,Izy6,Izz6}
    },
    {
      {Ixx7,Ixy7,Ixz7},
      {Iyx7,Iyy7,Iyz7},
      {Izx7,Izy7,Izz7}
    }
  };
#endif

    q = (double*)valptr_vn(jAng);
#if 0    
  /* Check for proper number of arguments. */
  if (nrhs != 1) {
    mexErrMsgTxt("One input required.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("Too many output arguments.");
  }
  
  /* The input must be a noncomplex double vector.*/
  mrows = mxGetM(prhs[0]);
  ncols = mxGetN(prhs[0]);
  if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) || !(ncols == 1)) {
    mexErrMsgTxt("Input must be a noncomplex double vector.");
  }

  dof = (mrows-1) / 3;

  /* Create matrix for the return argument. */
  plhs[0] = mxCreateDoubleMatrix(dof,1, mxREAL);

  /* Assign pointers to each input and output. */
  inp = mxGetPr(prhs[0]);
  out = mxGetPr(plhs[0]);


  for(i=0, j=dof, k=2*dof ; i<dof ; i++){
    tau[i] = inp[i];
      q[i] = inp[j++];
     dq[i] = inp[k++];
  }
#endif

  aux1=&MassMatrixVec[0];
  aux2=&CoriolisMatrixVec[0];
  aux3=&CoriolisTermVec[0];
  aux4=&GravityTermVec[0];

  for(i=0 ; i<dof ; i++){
    MassMatrix[i]=aux1;
    CoriolisMatrix[i]=aux2;
    CoriolisTerm[i]=aux3;
    GravityTerm[i]=aux4;
    aux1 += dof;
    aux2 += dof;
    aux3 += 1;
    aux4 += 1;
  }

/*clock_t tic1, tic2;
tic1=clock();*/
/*struct tms times1, times2;
times(&times1);*/

  #include "robot_Spec.h"

  ListMatrixIndex=0;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jvc1.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);

  ListMatrixIndex=1;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jvc2.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);

  ListMatrixIndex=2;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jvc3.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);

  ListMatrixIndex=3;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jvc4.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);

  ListMatrixIndex=4;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jtool.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);

#if 0
  ListMatrixIndex=4;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jvc5.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);

  ListMatrixIndex=5;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jvc6.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);

  ListMatrixIndex=6;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JvcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jvc7.c"
  gslJvc[ListMatrixIndex] = gsl_matrix_view_array(&JvcVec[aux], 3, dof);
#endif

  ListMatrixIndex=0;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JwcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jwc1.c"
  gslJwc[ListMatrixIndex] = gsl_matrix_view_array(&JwcVec[aux], 3, dof);

  ListMatrixIndex=1;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JwcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jwc2.c"
  gslJwc[ListMatrixIndex] = gsl_matrix_view_array(&JwcVec[aux], 3, dof);

  ListMatrixIndex=2;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JwcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jwc3.c"
  gslJwc[ListMatrixIndex] = gsl_matrix_view_array(&JwcVec[aux], 3, dof);

  ListMatrixIndex=3;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JwcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jwc4.c"
  gslJwc[ListMatrixIndex] = gsl_matrix_view_array(&JwcVec[aux], 3, dof);
  
  
#if 0
  ListMatrixIndex=4;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JwcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jwc5.c"
  gslJwc[ListMatrixIndex] = gsl_matrix_view_array(&JwcVec[aux], 3, dof);

  ListMatrixIndex=5;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JwcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jwc6.c"
  gslJwc[ListMatrixIndex] = gsl_matrix_view_array(&JwcVec[aux], 3, dof);

  ListMatrixIndex=6;
  ListMatrixRows=3;
  ListMatrixCols=dof;
  ListMatrixPtr=&JwcVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "Jwc7.c"
  gslJwc[ListMatrixIndex] = gsl_matrix_view_array(&JwcVec[aux], 3, dof);
#endif


  ListMatrixIndex=0;
  ListMatrixRows=4;
  ListMatrixCols=4;
  ListMatrixPtr=&ThomVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "T01.c"
  gslThom[ListMatrixIndex] = gsl_matrix_view_array(&ThomVec[aux], 4, 4);

  ListMatrixIndex=1;
  ListMatrixRows=4;
  ListMatrixCols=4;
  ListMatrixPtr=&ThomVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "T02.c"
  gslThom[ListMatrixIndex] = gsl_matrix_view_array(&ThomVec[aux], 4, 4);

  ListMatrixIndex=2;
  ListMatrixRows=4;
  ListMatrixCols=4;
  ListMatrixPtr=&ThomVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "T03.c"
  gslThom[ListMatrixIndex] = gsl_matrix_view_array(&ThomVec[aux], 4, 4);

  ListMatrixIndex=3;
  ListMatrixRows=4;
  ListMatrixCols=4;
  ListMatrixPtr=&ThomVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "T04.c"
  gslThom[ListMatrixIndex] = gsl_matrix_view_array(&ThomVec[aux], 4, 4);

  
#if 0
  ListMatrixIndex=4;
  ListMatrixRows=4;
  ListMatrixCols=4;
  ListMatrixPtr=&ThomVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "T05.c"
  gslThom[ListMatrixIndex] = gsl_matrix_view_array(&ThomVec[aux], 4, 4);

  ListMatrixIndex=5;
  ListMatrixRows=4;
  ListMatrixCols=4;
  ListMatrixPtr=&ThomVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "T06.c"
  gslThom[ListMatrixIndex] = gsl_matrix_view_array(&ThomVec[aux], 4, 4);

  ListMatrixIndex=6;
  ListMatrixRows=4;
  ListMatrixCols=4;
  ListMatrixPtr=&ThomVec[
    (aux=ListMatrixIndex*ListMatrixRows*ListMatrixCols)+
    (ListMatrixRows-1)*ListMatrixCols
  ];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "T07.c"
  gslThom[ListMatrixIndex] = gsl_matrix_view_array(&ThomVec[aux], 4, 4);
#endif


  gslMassMatrix = gsl_matrix_view_array(&MassMatrix[0][0], dof, dof);
  gsl_matrix_set_all(&gslMassMatrix.matrix,0.0);
  for(k=0 ; k<dof ; k++){
    /*gslThom[k] = gsl_matrix_view_array(&ThomVec[k*4*4], 4, 4);*/
    gslRot[k]  = gsl_matrix_view_array( &RotVec[k*3*3], 3, 3);
    /*--- gslRot = gslThom[0,i][[ Range[1,3],Range[1,3] ]] ---*/
    for(i=0 ; i<3 ; i++)
      for(j=0 ; j<3 ; j++)
        gsl_matrix_set(
          &gslRot[k].matrix,i,j,
          gsl_matrix_get(&gslThom[k].matrix,i,j)
        );
    /*--- auxm1 = Transpose[T[0,i][[ Range[1,3],Range[1,3] ]]] * Jwc[i] ---*/
    auxm1 = gsl_matrix_view_array(&auxm1Vec[0],3,dof);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,
      1.0,&gslRot[k].matrix,&gslJwc[k].matrix,
      0.0,&auxm1.matrix);
    /*--- auxm3 = inertia[i] * auxm1 ---*/
    auxm2 = gsl_matrix_view_array(&inertia[k][0][0],3,3);
    auxm3 = gsl_matrix_view_array(&auxm3Vec[0],3,dof);
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,
      1.0,&auxm2.matrix,&auxm1.matrix,
      0.0,&auxm3.matrix);
    /*--- auxm1 = T[0,i][[ Range[1,3],Range[1,3] ]] * auxm3 ---*/
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,
      1.0,&gslRot[k].matrix,&auxm3.matrix,
      0.0,&auxm1.matrix);
    /*--- auxm3 = Transpose[Jwc[i]] * auxm1 ---*/
    auxm3 = gsl_matrix_view_array(&auxm3Vec[0],dof,dof);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,
      1.0,&gslJwc[k].matrix,&auxm1.matrix,
      0.0,&auxm3.matrix);
    /*--- MassMatrix += auxm3 ---*/
    gsl_matrix_add(&gslMassMatrix.matrix,&auxm3.matrix);
    /*--- auxm2 = mass[i] Transpose[Jvc[i]].Jvc[i] ---*/
    auxm2 = gsl_matrix_view_array(&auxm2Vec[0],dof,dof);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,
      mass[k],&gslJvc[k].matrix,&gslJvc[k].matrix,
      0.0,&auxm2.matrix);
    /*--- MassMatrix += auxm2 ---*/
    gsl_matrix_add(&gslMassMatrix.matrix,&auxm2.matrix);

  }

/*
for(k=0 ; k<dof ; k++){
  mexPrintf("Jwc[%d]= ",k);
  for(i=0 ; i<3 ; i++){
    for(j=0 ; j<dof ; j++)
      mexPrintf("%f; ",gsl_matrix_get(&gslJwc[k].matrix,i,j));
    mexPrintf("\n");
  }
  mexPrintf("\n");
}
*/

/*
  ListMatrixRows=dof;
  ListMatrixCols=dof;
  ListMatrixPtr=&CoriolisMatrixVec[(ListMatrixRows-1)*ListMatrixCols];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "robot_CoriolisMatrix.c"
*/
  ListMatrixRows=1;
  ListMatrixCols=dof;
  ListMatrixPtr=&GravityTermVec[(ListMatrixRows-1)*ListMatrixCols];
  ListMatrixDPtr=&ListMatrixPtr;
  #include "robot_GravityTerm.c"

/*tic2=clock();
mexPrintf("Elapsed time = %f (tic1=%.1f ; tic2=%.1f ; CLOCKS_PER_SEC=%.1f).\n",
  (double) ((double) ( tic2 - tic1 ) / (double) CLOCKS_PER_SEC ),
  (double) tic1, (double) tic2, (double) CLOCKS_PER_SEC);*/
/*times(&times2);
mexPrintf("Elapsed time:\n  User=%f   (tic1=%.1f; tic2=%.1f);\n"
                         "  System=%f (tic1=%.1f; tic2=%.1f);\n"
                         "  Total=%f ;\n"
                         "  CLOCKS_PER_SEC=%.1f).\n",
  (double) ((double) ( times2.tms_utime - times1.tms_utime ) 
                     / (double) CLOCKS_PER_SEC ),
  (double) times1.tms_utime, (double) times2.tms_utime, 

  (double) ((double) ( times2.tms_stime - times1.tms_stime ) 
                     / (double) CLOCKS_PER_SEC ),
  (double) times1.tms_stime, (double) times2.tms_stime, 

  (double) ((double) ( times2.tms_utime - times1.tms_utime ) 
                     / (double) CLOCKS_PER_SEC )+
  (double) ((double) ( times2.tms_stime - times1.tms_stime ) 
                     / (double) CLOCKS_PER_SEC ),

  (double) CLOCKS_PER_SEC);*/

#if 0
for(i=0 ; i<dof ; i++)
  for(j=0 ; j<dof ; j++)
    mexPrintf("MassMatrix[%d][%d]=%f\n",i,j,MassMatrix[i][j]);

for(i=0 ; i<dof ; i++)
  for(j=0 ; j<dof ; j++)
    mexPrintf("CoriolisMatrix[%d][%d]=%f\n",i,j,CoriolisMatrix[i][j]);

for(i=0 ; i<dof ; i++)
  mexPrintf("GravityTerm[%d][0]=%f\n",i,GravityTerm[i][0]);
#endif

  for(i=0 ; i<dof ; i++)
    for(j=0 ; j<dof ; j++)
      //MassMatrix[i][j]=MassMatrix[j][i];
      setval_mn(M, i, j, MassMatrix[i][j]);

  // Khalil pg 201
  //ELEM(M, 0, 0) = ELEM(M, 0, 0) + J1motor;
  //ELEM(M, 1, 1) = ELEM(M, 1, 1) + J2motor;
  //ELEM(M, 2, 2) = ELEM(M, 2, 2) + J3motor;
  //ELEM(M, 3, 3) = ELEM(M, 3, 3) + J4motor;
  
  for(i = 0; i < 12; i++)
     Js->q[i] = JwcVec[36+i]; //Js->q[i] = JvcVec[48+i];	

  for(i = 0; i < dof; i++)
     setval_vn(G, i, GravityTerm[i][0]);
        
  for(i = 0; i < 16; i++)
     Th->q[i] = ThomVec[48+i];
  
  /*--- ddq=inv[D(q)]*[tau-C(q,dq)*dq-h(q)-b(q)]; b(q) not implemented. ---*/
  for(i=0 ; i<dof ; i++){
    CoriolisTerm[i][0] = 0.0;
    for(j=0 ; j<dof ; j++)
      CoriolisTerm[i][0] += CoriolisMatrix[i][j]*dq[j];
    ddq[i] = tau[i]-CoriolisTerm[i][0]-GravityTerm[i][0];
  }
  gslMassMatrix = gsl_matrix_view_array(&MassMatrix[0][0], dof, dof);
  b_side = gsl_vector_view_array(&ddq[0], dof);
  gsl_ddq = gsl_vector_alloc(dof);
  permut = gsl_permutation_alloc(dof);
  gsl_linalg_LU_decomp(&gslMassMatrix.matrix, permut, &signum);
  gsl_linalg_LU_solve(&gslMassMatrix.matrix, permut, &b_side.vector, gsl_ddq);
  gsl_permutation_free(permut);
  for(i=0 ; i<dof ; i++)
    ddq[i] = gsl_vector_get(gsl_ddq, i);
  gsl_vector_free(gsl_ddq);

#if 0
  for(i=0 ; i<dof ; i++)
    out[i] = ddq[i];
#endif

}

/*-----------------------------------------------------------------------*/
